import pytest
import requests
import numpy as np
from ezomero import json_api, store_connection_params
from ezomero import get_image_ids, get_image
from pathlib import Path


def test_connect_params(omero_params, tmp_path, monkeypatch):
    user, password, host, web_host, port, secure = omero_params

    # params should override environment variables
    monkeypatch.setenv("OMERO_USER", 'fail')
    monkeypatch.setenv("OMERO_PASS", 'fail')
    monkeypatch.setenv("OMERO_WEB_HOST", 'fail')

    # params should override config file
    conf_txt = ("[JSON]\n"
                "omero_user = fail\n"
                "omero_web_host = fail\n")
    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)

    login_rsp, session, base_url = json_api.create_json_session(
                                        user, password, web_host=web_host,
                                        config_path=str(tmp_path))
    r = session.get(base_url)
    assert login_rsp['success']
    assert base_url is not None
    assert r.status_code == 200


def test_connect_env(omero_params, tmp_path, monkeypatch):
    user, password, host, web_host, port, secure = omero_params
    monkeypatch.setenv("OMERO_USER", user)
    monkeypatch.setenv("OMERO_PASS", password)
    monkeypatch.setenv("OMERO_WEB_HOST", web_host)

    # env should override config file
    conf_txt = ("[JSON]\n"
                "omero_user = fail\n"
                "omero_web_host = fail\n")

    # test sanitizing input
    with pytest.raises(TypeError):
        login_rsp, session, base_url = json_api.create_json_session(
                                        config_path=100)

    conf_path = Path.home() / '.ezomero'
    conf_path.write_text(conf_txt)

    # test no input to config path defaulting to home
    login_rsp, session, base_url = json_api.create_json_session()
    assert login_rsp['success']

    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)
    login_rsp, session, base_url = json_api.create_json_session(
                                        config_path=str(tmp_path))
    assert login_rsp['success']
    with pytest.raises(requests.exceptions.HTTPError):
        login_rsp, session, base_url = json_api.create_json_session(
                                        user='fake_user')


def test_connect_config(omero_params, tmp_path):
    user, password, host, web_host, port, secure = omero_params
    conf_txt = ("[JSON]\n"
                f"omero_user = {user}\n"
                f"omero_web_host = {web_host}\n")
    conf_path = tmp_path / '.ezomero'
    conf_path.write_text(conf_txt)
    login_rsp, session, base_url = json_api.create_json_session(
                                        password=password,
                                        config_path=str(tmp_path))
    assert login_rsp['success']


def test_store_conn_params(omero_params, tmp_path):
    user, password, host, web_host, port, secure = omero_params
    store_connection_params(user=user, group="", host=host, port=port,
                            secure=True, web_host=web_host,
                            config_path=str(tmp_path))
    login_rsp, session, base_url = json_api.create_json_session(
                                        password=password,
                                        config_path=str(tmp_path))
    assert login_rsp['success']


def test_get_rendered_jpegs(omero_params, conn, pyramid_fixture):
    user, password, host, web_host, port, secure = omero_params
    pyr_id = get_image_ids(conn)[-1]
    img, pix = get_image(conn, pyr_id)
    login_rsp, session, base_url = json_api.create_json_session(
                                        user, password, web_host=web_host)
    with pytest.raises(TypeError):
        jpeg = json_api.get_rendered_jpeg('test', base_url, pyr_id, 1)
    with pytest.raises(TypeError):
        jpeg = json_api.get_rendered_jpeg(session, 10, pyr_id, 1)
    with pytest.raises(TypeError):
        jpeg = json_api.get_rendered_jpeg(session, base_url, 'test', 1)
    with pytest.raises(TypeError):
        jpeg = json_api.get_rendered_jpeg(session, base_url, pyr_id, '1')

    with pytest.raises(requests.exceptions.HTTPError):
        jpeg = json_api.get_rendered_jpeg(session, base_url, 99999, 1)
    with pytest.raises(requests.exceptions.ConnectionError):
        jpeg = json_api.get_rendered_jpeg(session,
                                          'http://messedupbaseurl', 99999, 1)

    jpeg = json_api.get_rendered_jpeg(session, base_url, pyr_id, 1)
    assert np.shape(pix)[2] == np.shape(jpeg)[0]  # xdim = xdim
    assert np.shape(pix)[3] == np.shape(jpeg)[1]  # ydim = ydim
    assert np.shape(jpeg)[2] == 3  # RGB image

    jpeg = json_api.get_rendered_jpeg(session, base_url, pyr_id, 3.0)
    assert round(np.shape(pix)[2]/3) == np.shape(jpeg)[0]
    assert round(np.shape(pix)[3]/3) == np.shape(jpeg)[1]
    assert np.shape(jpeg)[2] == 3



from dataclasses import dataclass, field
from typing import List, Tuple, Union

__all__ = ["Point",
           "Line",
           "Rectangle",
           "Ellipse",
           "Polygon",
           "Polyline",
           "Label",
           "ezShape"]


@dataclass(frozen=True)
class ezShape:
    """Generic dataclass used to create an OMERO Shape.

    A dataclass used to represent any shape and create an OMERO equivalent.
    This class has no fields; it's just extended by all ROIs.
    This dataclass is frozen and should not be modified after instantiation

    """


@dataclass(frozen=True)
class Point(ezShape):
    """A dataclass used to create an OMERO Point.

    A dataclass used to represent a Point shape and create an OMERO equivalent.
    This dataclass is frozen and should not be modified after instantiation

    Parameters
    ----------
    x : float
        The x axis position of the point shape in pixels.
    y : float
        The y axis position of the point shape in pixels.
    z : int, optional
        The z position of the point in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the Point will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the Point will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Line(ezShape):
    """A dataclass used to create an OMERO Line.

    A dataclass used to represent a Line shape and create an OMERO equivalent.
    This dataclass is frozen and should not be modified after instantiation.

    Parameters
    ----------
    x1 : float
        The x axis position of the start point of the line shape in pixels.
    y1 : float
        The y axis position of the start point of the line shape in pixels.
    x2 : float
        The x axis position of the end point of the line shape in pixels.
    y2 : float
        The y axis position of the end point of the line shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    markerStart : str, optional
        The marker for the start of the line. Default is ``None``.
    markerEnd : str, optional
        The marker for the end of the line. Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    x1: float = field(metadata={'units': 'PIXELS'})
    y1: float = field(metadata={'units': 'PIXELS'})
    x2: float = field(metadata={'units': 'PIXELS'})
    y2: float = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    markerStart: Union[str, None] = field(default=None)
    markerEnd: Union[str, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Rectangle(ezShape):
    """A dataclass used to create an OMERO rectangle.

    A dataclass used to represent a Rectangle shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    x : float
        The x axis position of the rectangle shape in pixels.
    y : float
        The y axis position of the rectangle shape in pixels.
    width : float
        The width (x axis) of the rectangle shape in pixels.
    height : float
        The height (y axis) of the rectangle shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    width: float = field(metadata={'units': 'PIXELS'})
    height: float = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Ellipse(ezShape):
    """A dataclass used to create an OMERO Ellipse.

    A dataclass used to represent an Ellipse shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    x : float
        The x axis position of the ellipse shape in pixels.
    y : float
        The y axis position of the ellipse shape in pixels.
    x_rad : float
        The x radius of the ellipse shape in pixels.
    y_rad : float
        The y radius of the ellipse shape in pixels.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    x_rad: float = field(metadata={'units': 'PIXELS'})
    y_rad: float = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Polygon(ezShape):
    """A dataclass used to create an OMERO polygon.

    A dataclass used to represent a Polygon shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    points : list of tuples of 2 floats
        A list of 2 element tuples corresponding to the (x, y) coordinates of
        each vertex of the polygon.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    points: List[Tuple[float, float]] = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Polyline(ezShape):
    """A dataclass used to create an OMERO polyline.

    A dataclass used to represent a Polyline shape and create an OMERO
    equivalent. This dataclass is frozen and should not be modified after
    instantiation.

    Parameters
    ----------
    points : list of tuples of 2 floats
        A list of 2 element tuples corresponding to the (x, y) coordinates of
        each vertex of the polyline.
    z : int, optional
        The z position of the shape in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Point will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the shape will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the shape will not be linked to any time frame.
        Default is ``None``.
    label : str, optional
        The label of the shape. Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    points: List[Tuple[float, float]] = field(metadata={'units': 'PIXELS'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    label: Union[str, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)


@dataclass(frozen=True)
class Label(ezShape):
    """A dataclass used to create an OMERO Label.

    A dataclass used to represent a Label shape and create an OMERO equivalent.
    This dataclass is frozen and should not be modified after instantiation

    Parameters
    ----------
    x : float
        The x axis position of the label shape in pixels.
    y : float
        The y axis position of the label shape in pixels.
    label : str
        The text value of the Label.
    fontSize: int
        The font size of the label, in pt.
    z : int, optional
        The z position of the label in pixels. Note this is the z plane to
        which the shape is linked and not the sub-voxel resolution position of
        your shape. If ``None``, the Label will not be linked to any z plane.
        Default is ``None``.
    c : int, optional
        The channel index to which the shape is linked.
        If None, the Label will not be linked to any channel. Default is
        ``None``.
    t : int, optional
        The time frame to which the shape is linked.
        If ``None``, the Label will not be linked to any time frame.
        Default is ``None``.
    fill_color : Tuple (4 integers), optional
        The color that will be used as the fill/background of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (0, 0, 0, 0) (i.e. transparent) in
        ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_color : Tuple (4 integers), optional
        The color that will be used as the stroke/perimeter of this Shape.
        Format is RGBA, 0-255.
        If ``None``, we default to (255, 255, 0, 255) (i.e. yellow, following
        OMERO.iViewer default) in ``ezomero.get_shape()``.
        Default is ``None``.
    stroke_width : float, optional
        The line width used for the stroke/perimeter of this Shape.
        If ``None``, we default to 1.0 in ``ezomero.get_shape()``.
        Default is ``None``.
    """

    x: float = field(metadata={'units': 'PIXELS'})
    y: float = field(metadata={'units': 'PIXELS'})
    label: str = field()
    fontSize: int = field(metadata={'FontSizeUnit': 'pt'})
    z: Union[int, None] = field(default=None)
    c: Union[int, None] = field(default=None)
    t: Union[int, None] = field(default=None)
    fill_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_color: Union[Tuple[int, int, int, int], None] = field(default=None)
    stroke_width: Union[float, None] = field(default=None)



import pytest
import ezomero

# Test puts
###########


def test_put_map_annotation(conn, project_structure, users_groups):
    kv = {"key1": "value1",
          "key2": "value2"}
    ns = "jax.org/omeroutils/tests/v0"

    # test sanitized input
    with pytest.raises(TypeError):
        _ = ezomero.put_map_annotation(conn, '10', kv)
    with pytest.raises(ValueError):
        _ = ezomero.put_map_annotation(conn, 99999999, kv)

    image_info = project_structure[2]
    im_id = image_info[0][1]
    map_ann_id = ezomero.post_map_annotation(conn, "Image", im_id, kv, ns)
    kv = {"key1": "changed1",
          "key2": "value2"}
    ezomero.put_map_annotation(conn, map_ann_id, kv)
    kv_pairs = ezomero.get_map_annotation(conn, map_ann_id)
    assert kv_pairs['key1'] == kv['key1']

    # test cross-group
    kv = {"key1": "value1",
          "key2": "value2"}
    username = users_groups[1][0][0]  # test_user1
    groupname = users_groups[0][0][0]  # test_group_1
    current_conn = conn.suConn(username, groupname)
    im_id2 = image_info[2][1]  # im2, in test_group_2
    map_ann_id2 = ezomero.post_map_annotation(current_conn, "Image", im_id2,
                                              kv, ns)
    print(map_ann_id2)
    kv = {"key1": "changed1",
          "key2": "value2"}
    ezomero.put_map_annotation(current_conn, map_ann_id2, kv)
    kv_pairs = ezomero.get_map_annotation(current_conn, map_ann_id2)
    assert kv_pairs['key1'] == kv['key1']
    current_conn.close()

    # test cross-group, across_groups unset
    kv = {"key1": "value1",
          "key2": "value2"}
    username = users_groups[1][0][0]  # test_user1
    groupname = users_groups[0][0][0]  # test_group_1
    current_conn = conn.suConn(username, groupname)
    im_id3 = image_info[2][1]  # im2, in test_group_2
    map_ann_id3 = ezomero.post_map_annotation(current_conn, "Image", im_id3,
                                              kv, ns)
    print(map_ann_id3)
    kv_changed = {"key1": "changed1",
                  "key2": "value2"}
    with pytest.raises(ValueError):
        ezomero.put_map_annotation(current_conn, map_ann_id3, kv_changed,
                                   across_groups=False)
    kv_pairs = ezomero.get_map_annotation(current_conn, map_ann_id3)
    assert kv_pairs['key1'] == kv['key1']
    current_conn.close()

    # test non-existent ID
    with pytest.raises(ValueError):
        ezomero.put_map_annotation(conn, 9999999, kv)

    conn.deleteObjects("Annotation",
                       [map_ann_id, map_ann_id2],
                       deleteAnns=True,
                       deleteChildren=True,
                       wait=True)


def test_put_description(conn, project_structure, users_groups):
    desc = "test description"

    # test sanitized input
    with pytest.raises(TypeError):
        _ = ezomero.put_description(conn, 10, 'Image', desc)
    with pytest.raises(TypeError):
        _ = ezomero.put_description(conn, 'Image', '10', desc)
    with pytest.raises(ValueError):
        _ = ezomero.put_description(conn, 'FakeImage', 1, desc)

    image_info = project_structure[2]
    im_id = image_info[0][1]
    ezomero.put_description(conn, 'Image', im_id, desc)
    img, _ = ezomero.get_image(conn, im_id, no_pixels=True)
    assert img.getDescription() == desc

    # test cross-group
    username = users_groups[1][0][0]  # test_user1
    groupname = users_groups[0][0][0]  # test_group_1
    current_conn = conn.suConn(username, groupname)
    im_id2 = image_info[2][1]  # im2, in test_group_2
    ezomero.put_description(conn, 'Image', im_id2, desc)
    img, _ = ezomero.get_image(conn, im_id2, no_pixels=True)
    assert img.getDescription() == desc
    current_conn.close()

    # test cross-group, across_groups unset
    username = users_groups[1][0][0]  # test_user1
    groupname = users_groups[0][0][0]  # test_group_1
    current_conn = conn.suConn(username, groupname)
    im_id3 = image_info[2][1]  # im2, in test_group_2
    ezomero.put_description(conn, 'Image', im_id3, desc)
    img, _ = ezomero.get_image(conn, im_id3, no_pixels=True)
    assert img.getDescription() == desc
    current_conn.close()

    # test non-existent ID
    with pytest.raises(ValueError):
        ezomero.put_description(conn, 'Image', 9999999, desc)



import ezomero
from io import StringIO

# Test imports


def test_ezimport(conn, monkeypatch):

    # test simple import, single file
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath)
    assert len(id) == 1
    conn.deleteObjects("Image", id)

#     # test simple import, multifile/multi-image
    fpath = "tests/data/vsi-ets-test-jpg2k.vsi"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath)
    assert len(id) == 2

#     # test simple import, new orphan dataset
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath, dataset="test_ds")
    assert len(id) == 1
    ds_id = ezomero.get_dataset_ids(conn)[-1]
    im_ids = ezomero.get_image_ids(conn, dataset=ds_id)
    assert len(im_ids) == 1
    assert im_ids[0] == id[-1]

#     # test simple import, existing dataset
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath, dataset=ds_id)
    im_ids = ezomero.get_image_ids(conn, dataset=ds_id)
    assert len(im_ids) == 2
    assert im_ids[-1] == id[-1]
    conn.deleteObjects("Dataset", [ds_id], deleteChildren=True)

#     # test simple import, new project
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath,
                          project="test_proj", dataset="test_ds")
    assert len(id) == 1
    proj_id = ezomero.get_project_ids(conn)[-1]
    ds_id = ezomero.get_dataset_ids(conn, project=proj_id)[-1]
    im_ids = ezomero.get_image_ids(conn, dataset=ds_id)
    assert len(im_ids) == 1
    assert im_ids[0] == id[-1]

#     # test simple import, existing project, new dataset
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath,
                          project=proj_id, dataset="new_test_ds")
    ds_ids = ezomero.get_dataset_ids(conn, project=proj_id)
    im_ids = ezomero.get_image_ids(conn, dataset=ds_ids[-1])
    assert len(ds_ids) == 2
    assert len(im_ids) == 1
    assert im_ids[0] == id[-1]

#     # test simple import, existing project, existing dataset
    fpath = "tests/data/test_pyramid.ome.tif"
    str_input = ["omero", 'import',
                 '-k', conn.getSession().getUuid().val,
                 '-s', conn.host,
                 '-p', str(conn.port),
                 fpath, "\n"]
    io = StringIO(" ".join(str_input))
    monkeypatch.setattr('sys.stdin', io)
    id = ezomero.ezimport(conn, fpath,
                          project=proj_id, dataset=ds_id)
    ds_id = ezomero.get_dataset_ids(conn, project=proj_id)[0]
    im_ids = ezomero.get_image_ids(conn, dataset=ds_id)
    assert len(im_ids) == 2
    assert im_ids[-1] == id[-1]

    conn.deleteObjects("Project", [proj_id], deleteChildren=True)



import setuptools
import os

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ezomero",
    version=os.environ.get('VERSION', '0.0.0'),
    maintainer="Dave Mellert",
    maintainer_email="Dave.Mellert@jax.org",
    description=("A suite of convenience functions for working"
                 " with OMERO. Written and maintained by the "
                 "Research IT team at The Jackson Laboratory."),
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TheJacksonLaboratory/ezomero",
    packages=setuptools.find_packages(),
    install_requires=[
        'omero-py >= 5.13.0, < 5.20.0',
        'numpy >= 1.22, < 2.0'
    ],
    extras_require={
        "tables": ["pandas"],
    },
    python_requires='>=3.8'
)
